/*
  -------------------------------------------------------------------
  
  Copyright (C) 2012, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/vector_derint.h>
#include <o2scl/columnify.h>
#include <o2scl/test_mgr.h>

using namespace std;
using namespace o2scl;

int main(void) {
  cout.setf(ios::scientific);

  test_mgr t;
  t.set_output_level(1);

  static const size_t n=50;

  ovector sinx(n), cosx(n);
  for(size_t i=0;i<n;i++) {
    sinx[i]=sin(((double)i)/4.0);
    cosx[i]=cos(((double)i)/4.0);
  }

  def_interp_mgr<ovector,cspline_interp> csp;
  def_interp_mgr<ovector,akima_interp> akm;

  {
    ovector t1(n), t2(n), t3(n), t4(n), t5(n), t6(n);
    vector_deriv_threept(n,sinx,t1);
    vector_deriv_threept_tap(n,sinx,t2);
    vector_deriv_fivept(n,sinx,t3);
    vector_deriv_fivept_tap(n,sinx,t4);
    vector_deriv_interp(n,sinx,t5,csp);
    vector_deriv_interp(n,sinx,t6,akm);

    cout << "Differentiation: " << endl;
    cout.setf(ios::showpos);
    cout.precision(4);
    for(size_t i=0;i<n;i++) {
      cout << fabs(t1[i]-0.25*cosx[i]) << " " 
	   << fabs(t2[i]-0.25*cosx[i]) << " " 
	   << fabs(t3[i]-0.25*cosx[i]) << " " 
	   << fabs(t4[i]-0.25*cosx[i]) << " " 
	   << fabs(t5[i]-0.25*cosx[i]) << " " 
	   << fabs(t6[i]-0.25*cosx[i]) << endl;
    }
    cout << endl;
    cout.precision(6);
    cout.unsetf(ios::showpos);
  }
  
  cout << "Integration: " << endl;
  double exact=(1.0-cos(((double)(n-1))/4.0));
  cout << fabs(exact-0.25*vector_integ_trap(n,sinx)) << endl;
  cout << fabs(exact-0.25*vector_integ_threept(n,sinx)) << endl;
  cout << fabs(exact-0.25*vector_integ_extended4(n,sinx)) << endl;
  cout << fabs(exact-0.25*vector_integ_durand(n,sinx)) << endl;
  cout << fabs(exact-0.25*vector_integ_extended8(n,sinx)) << endl;
  cout << fabs(exact-0.25*vector_integ_interp(n,sinx,csp)) << endl;
  cout << fabs(exact-0.25*vector_integ_interp(n,sinx,akm)) << endl;
  
  t.report();
  return 0;
}

