/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006-2012, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <iostream>
#include <o2scl/test_mgr.h>
#include <o2scl/constants.h>
#include <o2scl/gen_sn_eos.h>

using namespace std;
using namespace o2scl;
using namespace o2scl_const;

int main(int argv, char *argc[]) {
  
  cout.setf(ios::scientific);
  cout.precision(3);
  
  test_mgr t;
  t.set_output_level(1);
  
  if (argv>=3) {
    
    string mode=argc[1];
    string fname=argc[2];
    
    double cf=(gsl_cgs::mass_neutron+gsl_cgs::mass_proton)/2.0*1.0e39;
    
    if (mode=="ls") {
      
      ls_eos ls;  
      ls.verbose=2;
      ls.load(fname);
      ls.check_eg(t);
      ls.check_free_energy(t);

      double nb, ye, T;
    
      cout << "Look at the equilibrium nuclei for nearly zero T and Ye=0.5:" 
	   << endl;
      cout << "rho       Ye        T         "
	   << "Z         A         N_skin: " << endl;
      for(double rho=1.0e9;rho<=1.0e15;rho*=2.0) {
	cout << rho << " " << 0.5 << " " << 0.32 << " "
	     << ls.Z.interp(rho/cf,0.5,0.32) << " "
	     << ls.A.interp(rho/cf,0.5,0.32) << " "
	     << ls.Nskin.interp(rho/cf,0.5,0.32) << endl;
      }
      cout << endl;

    } else if (mode=="oo") {
      
      oo_eos oo;  
      oo.verbose=2;
      
      if (argc[4]=="ls") {
	oo.load(fname,0);
      } else {
	oo.load(fname,1);
      }

    } else if (mode=="stos") {

      stos_eos stos;
      stos.load(fname);

      double nb, ye, T;
    
      cout << "Look at the equilibrium nuclei for nearly zero T and Ye=0.5:" 
	   << endl;
      cout << "rho       Ye        T         "
	   << "Z         A         X_nuclei: " << endl;
      for(double rho=1.0e9;rho<=1.0e15;rho*=2.0) {
	cout << rho << " " << 0.5 << " " << 0.32 << " "
	     << stos.Z.interp(rho/cf,0.5,0.32) << " "
	     << stos.A.interp(rho/cf,0.5,0.32) << " "
	     << stos.Xnuclei.interp(rho/cf,0.5,0.32) << endl;
      }
      cout << endl;

    } else if (mode=="sht") {

      string fname2=argc[3];
      sht_eos sht, sht2;
      int imode=stoi(argc[4]);
      sht.load(fname,imode);
      sht2.load(fname2,imode+2);

      //gen_sn_eos_ts sht $(HOME)/pkgs/sht_eos/FSU2.1eos1.01.dat 
      //$(HOME)/pkgs/sht_eos/FSU2.1eosb1.01.dat 1		 
      //>> gen_sn_test.scr

      double nb, ye, T;
    
      if (false) {
	cout << "Look at the equilibrium nuclei for nearly zero T and Ye=0.5:" 
	     << endl;
	cout << "rho       Ye        T         "
	     << "Z         A         X_nuclei: " << endl;
	for(double rho=1.0e9;rho<=1.0e15;rho*=2.0) {
	  cout << rho << " " << 0.5 << " " << 0.32 << " "
	       << sht.Z.interp(rho/cf,0.5,0.32) << " "
	       << sht.A.interp(rho/cf,0.5,0.32) << " "
	       << sht.Xnuclei.interp(rho/cf,0.5,0.32) << endl;
	}
	cout << endl;
	
	cout << "Look at the equilibrium nuclei for nearly zero T and Ye=0.5:" 
	     << endl;
	cout << "rho       Ye        T         "
	     << "Z         A         X_nuclei: " << endl;
	for(double rho=1.0e9;rho<=1.0e15;rho*=2.0) {
	  cout << rho << " " << 0.5 << " " << 0.32 << " "
	       << sht2.Z.interp(rho/cf,0.5,0.32) << " "
	       << sht2.A.interp(rho/cf,0.5,0.32) << " "
	       << sht2.Xnuclei.interp(rho/cf,0.5,0.32) << endl;
	}
	cout << endl;
      }

      {
	cout << "Checking electrons and photons:" << endl;
	
	int i=10, j=10, k=10;
	
	t.set_output_level(0);
	
	for(size_t ell=0;ell<40;ell++) {
	  if (ell%3==0) {
	    i+=49;
	    i=i%sht.n_nB;
	  }
	  if (ell%3==1) {
	    j+=49;
	    j=j%sht.n_Ye;
	  }
	  if (ell%3==2) {
	    k+=49;
	    k=k%sht.n_T;
	  }
	  double nb1, ye1, T1;
	  nb1=sht.E.get_grid(0,i);
	  ye1=sht.E.get_grid(1,j);
	  T1=sht.E.get_grid(2,k);

	  if (ye1>0.0) {

	    sht.electron.n=nb1*ye1;
	    cout << nb1 << " " << ye1 << " " << T1 << " " << std::flush;
	    sht.relf.pair_density(sht.electron,T1/hc_mev_fm);
	    sht.photon.massless_calc_mu(T1/hc_mev_fm);
	    
	    double E_eg=(sht.electron.ed+sht.photon.ed)/nb1*hc_mev_fm;
	    double val=sht.E.interp(nb1,ye1,T1)-sht2.Eint.interp(nb1,ye1,T1);
	    double P_eg=(sht.electron.pr+sht.photon.pr)*hc_mev_fm;
	    double val2=sht.P.interp(nb1,ye1,T1)-sht2.Pint.interp(nb1,ye1,T1);
	    double S_eg=(sht.electron.en+sht.photon.en)/nb1;
	    double val3=sht.S.interp(nb1,ye1,T1)-sht2.Sint.interp(nb1,ye1,T1);

	    // Currently, the entropy doesn't match so well at low
	    // temperatures and high densities and the pressure
	    // doesn't match so well at high temperature and low
	    // densities. 
	    cout << fabs(val-E_eg)/fabs(val) << " ";
	    cout << fabs(val2-P_eg)/fabs(val2) << " ";
	    cout << fabs(val3-S_eg)/fabs(val3) << endl;
	    if (fabs(val2-P_eg)/fabs(val2)>1.0e-2) {
	      cout << "\tP " << sht.P.interp(nb1,ye1,T1) << " " 
		   << sht2.Pint.interp(nb1,ye1,T1) << " " << P_eg << endl;
	    }
	    if (fabs(val3-S_eg)/fabs(val3)>1.0e-2) {
	      cout << "\tS " << sht.S.interp(nb1,ye1,T1) << " " 
		   << sht2.Sint.interp(nb1,ye1,T1) << " " << S_eg << endl;
	    }
	    t.test_rel(fabs(val-E_eg)/fabs(val),0.0,1.0e-2,"e and g");
	    t.test_rel(fabs(val2-P_eg)/fabs(val2),0.0,4.0e-2,"e and g");
	    t.test_rel(fabs(val3-S_eg)/fabs(val3),0.0,1.0e-1,"e and g");
	  }
	       
	}
	
	if (t.report()==false) {
	  O2SCL_ERR("Function sht_eos::load() failed.",gsl_efailed);
	}
	
      }
      
    } else if (mode=="hfsl") {

      hfsl_eos hfsl;
      hfsl.load(fname);

      double nb, ye, T;
    
      cout << "Look at the equilibrium nuclei for nearly zero T and Ye=0.5:" 
	   << endl;
      cout << "rho       Ye        T         "
	   << "Z         A         X_nuclei: " << endl;
      for(double rho=1.0e9;rho<=1.0e15;rho*=2.0) {
	cout << rho << " " << 0.5 << " " << 0.32 << " "
	     << hfsl.Z.interp(rho/cf,0.5,0.32) << " "
	     << hfsl.A.interp(rho/cf,0.5,0.32) << " "
	     << hfsl.Xnuclei.interp(rho/cf,0.5,0.32) << endl;
      }
      cout << endl;

    }

  }
    
  t.report();

  return 0;
}


