/*
  -------------------------------------------------------------------

  Copyright (C) 2006-2012, Andrew W. Steiner

  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifndef O2SCL_HDF_EOS_IO_H
#define O2SCL_HDF_EOS_IO_H

/** \file hdf_eos_io.h
    \brief File for HDF input of the \o2 \ref skyrme_eos and \ref
    rmf_eos data files
*/

#include <hdf5.h>

#include <o2scl/constants.h>
#include <o2scl/hdf_file.h>
#include <o2scl/lib_settings.h>

#include <o2scl/skyrme_eos.h>
#include <o2scl/rmf_eos.h>

#ifndef DOXYGENP
namespace o2scl_hdf {
#endif

#ifdef DOXYGENP
  // Separately document these functions since they're in the hidden
  // o2scl namespace

  /// Input a \ref rmf_eos object from an HDF file
  int rmf_load(rmf_eos &rmf, std::string model, bool external=false);

  /// Input a \ref skyrme_eos object from an HDF file
  int skyrme_load(skyrme_eos &sk, std::string model, bool external=false);

#else
  int rmf_load(o2scl::rmf_eos &rmf, std::string model, 
	       bool external=false);

  int skyrme_load(o2scl::skyrme_eos &sk, std::string model, 
		  bool external=false);
#endif
  
#ifndef DOXYGENP
}
#endif

#endif


