/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006-2012, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifndef O2SCL_TOV_EOS_FAST_H
#define O2SCL_TOV_EOS_FAST_H

#include <cmath>
#include <iostream>
#include <fstream>

#include <o2scl/tov_eos.h>

#ifndef DOXYGENP
namespace o2scl {
#endif

  /** \brief An EOS 
  */
  class tov_eos_fast : public tov_eos {
    
  public:
    
    tov_eos_fast();

    virtual ~tov_eos_fast();

    /// Given the pressure, produce the energy and number densities
    virtual int get_eden(double pres, double &ed, double &nb);

    /// Desc
    int set_crust_core(size_t n_crust, ovector_base &crust_e, 
		       ovector_base &crust_p, ovector_base &crust_nb, 
		       size_t n_core, ovector_base &core_e, 
		       ovector_base &core_p, ovector_base &core_nb);

    /** \brief If true, check the EOS for stability and ordering
	(default true)
    */
    bool check_eos;

#ifndef DOXYGEN_INTERNAL

  protected:

    /// Desc
    size_t nlines;

    /// Desc
    ovector edv;

    /// Desc
    ovector prv;

    /// Desc
    ovector nbv;

    /// Desc
    bool eos_set;

    /// Linear EOS interpolation
    void interp(const ovector_base &x, const ovector_base &y, 
		double xx, double &yy, int n1, int n2);

#endif

  };

#ifndef DOXYGENP
}
#endif

#endif


