/*
  -------------------------------------------------------------------

  Copyright (C) 2006-2012, Andrew W. Steiner

  This file is part of O2scl.

  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.

  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <gsl/gsl_blas.h>
#include <o2scl/test_mgr.h>
#include <o2scl/ovector_tlate.h>
#include <o2scl/omatrix_tlate.h>
#include <o2scl/cblas.h>
#include <o2scl/columnify.h>

using namespace std;
using namespace o2scl;

int main(void) {
  test_mgr t;
  t.set_output_level(1);

  ovector v1(5), v2(5), v3(5);
  ovector g1(5), g2(5), g3(5);
  ovector h1(5), h2(5), h3(5);
  omatrix m1(5,5), gm1(5,5), hm1(5,5);
  omatrix m2(5,5), gm2(5,5), hm2(5,5);
  omatrix m3(5,5), gm3(5,5), hm3(5,5);
  double a1[5], a2[5], a3[5];
  double dm1[5][5];
  double dm2[5][5];
  double dm3[5][5];
  double res1, res2, res3, res4;

  // Test with operator[]
  {
    cout << "Test with operator[]: " << endl;

    for(size_t i=0;i<5;i++) {
      a1[i]=sin((double)(i+1));
      a2[i]=cos((double)(i+1));
      a3[i]=tan((double)(i+1));
      v1[i]=sin((double)(i+1));
      v2[i]=cos((double)(i+1));
      v3[i]=tan((double)(i+1));
      g1[i]=sin((double)(i+1));
      g2[i]=cos((double)(i+1));
      g3[i]=tan((double)(i+1));
      h1[i]=sin((double)(i+1));
      h2[i]=cos((double)(i+1));
      h3[i]=tan((double)(i+1));
      for(size_t j=0;j<5;j++) {
	dm1[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
	m1[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
	gm1[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
	hm1[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
	dm2[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
	m2[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
	gm2[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
	hm2[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
	dm3[i][j]=0.0;
	m3[i][j]=0.0;
	gm3[i][j]=0.0;
	hm3[i][j]=0.0;
      }
    }
  
    // daxpy
    cout << "daxpy: " << endl;
    gsl_blas_daxpy(0.5,&g1,&g2);
    cblas_daxpy(5,0.5,h1.data,1,h2.data,1);
    o2scl_cblas::daxpy(0.5,5,v1,v2);
    o2scl_cblas::daxpy(0.5,5,a1,a2);
    t.test_rel_arr(5,g2,h2,1.0e-12,"daxpy1 operator[]");
    t.test_rel_arr(5,v2,h2,1.0e-12,"daxpy2 operator[] ");
    t.test_rel_arr(5,a2,h2,1.0e-12,"daxpy2 operator[] ");

    // ddot
    cout << "ddot: " << endl;
    gsl_blas_ddot(&g1,&g2,&res1);
    res2=cblas_ddot(5,h1.data,1,h2.data,1);
    res3=o2scl_cblas::ddot(5,v1,v2);
    res4=o2scl_cblas::ddot(5,a1,a2);
    t.test_rel(res1,res2,1.0e-12,"ddot1 operator[] ");
    t.test_rel(res1,res3,1.0e-12,"ddot2 operator[] ");
    t.test_rel(res1,res4,1.0e-12,"ddot2 operator[] ");
  
    // dnrm2
    cout << "dnrm2: " << endl;
    res1=gsl_blas_dnrm2(&g1);
    res2=cblas_dnrm2(5,h1.data,1);
    res3=o2scl_cblas::dnrm2(5,v1);
    res4=o2scl_cblas::dnrm2(5,a1);
    t.test_rel(res1,res2,1.0e-12,"dnrm21 operator[] ");
    t.test_rel(res1,res3,1.0e-12,"dnrm22 operator[] ");
    t.test_rel(res1,res4,1.0e-12,"dnrm22 operator[] ");
  
    // dscal
    cout << "dscal: " << endl;
    gsl_blas_dscal(0.5,&g3);
    cblas_dscal(5,0.5,h3.data,1);
    o2scl_cblas::dscal(0.5,5,v3);
    o2scl_cblas::dscal(0.5,5,a3);
    t.test_rel_arr(5,g3,h3,1.0e-12,"dscal1 operator[] ");
    t.test_rel_arr(5,v3,h3,1.0e-12,"dscal2 operator[] ");
    t.test_rel_arr(5,a3,h3,1.0e-12,"dscal2 operator[] ");

    // dgemv, NoTrans
    cout << "dgemv, NoTrans: " << endl;
    gsl_blas_dgemv(CblasNoTrans,0.1,&gm1,&g1,0.2,&g2);
    cblas_dgemv(CblasRowMajor,CblasNoTrans,5,5,0.1,hm1.data,5,
		h1.data,1,0.2,h2.data,1);
    o2scl_cblas::dgemv(o2scl_cblas::o2cblas_RowMajor,
		       o2scl_cblas::o2cblas_NoTrans,
		       5,5,0.1,m1,v1,0.2,v2);
    o2scl_cblas::dgemv(o2scl_cblas::o2cblas_RowMajor,
		       o2scl_cblas::o2cblas_NoTrans,
		       5,5,0.1,dm1,a1,0.2,a2);
    t.test_rel_arr(5,g2,h2,1.0e-12,"dgemv1 operator[] ");
    t.test_rel_arr(5,v2,h2,1.0e-12,"dgemv2 operator[] ");
    t.test_rel_arr(5,a2,h2,1.0e-12,"dgemv2 operator[] ");

    // dgemv, Trans (Note that ConjTrans was not originally supported)
    cout << "dgemv, Trans: " << endl;
    gsl_blas_dgemv(CblasTrans,0.1,&gm1,&g1,0.2,&g2);
    cblas_dgemv(CblasRowMajor,CblasTrans,5,5,0.1,hm1.data,5,
		h1.data,1,0.2,h2.data,1);
    o2scl_cblas::dgemv(o2scl_cblas::o2cblas_RowMajor,
		       o2scl_cblas::o2cblas_Trans,
		       5,5,0.1,m1,v1,0.2,v2);
    o2scl_cblas::dgemv(o2scl_cblas::o2cblas_RowMajor,
		       o2scl_cblas::o2cblas_Trans,
		       5,5,0.1,dm1,a1,0.2,a2);
    t.test_rel_arr(5,g2,h2,1.0e-12,"dgemv1t operator[] ");
    t.test_rel_arr(5,v2,h2,1.0e-12,"dgemv2t operator[] ");
    t.test_rel_arr(5,a2,h2,1.0e-12,"dgemv2t operator[] ");

    // dtrsv, Upper, NoTrans, NonUnit
    cout << "dtrsv, Upper, NoTrans, NonUnit: " << endl;
    gsl_blas_dtrsv(CblasUpper,CblasNoTrans,CblasNonUnit,&gm1,&g3);
    cblas_dtrsv(CblasRowMajor,CblasUpper,CblasNoTrans,CblasNonUnit,
		5,hm1.data,5,h3.data,1);
    o2scl_cblas::dtrsv(o2scl_cblas::o2cblas_RowMajor,
		       o2scl_cblas::o2cblas_Upper,
		       o2scl_cblas::o2cblas_NoTrans,
		       o2scl_cblas::o2cblas_NonUnit,5,5,m1,v3);
    o2scl_cblas::dtrsv(o2scl_cblas::o2cblas_RowMajor,
		       o2scl_cblas::o2cblas_Upper,
		       o2scl_cblas::o2cblas_NoTrans,
		       o2scl_cblas::o2cblas_NonUnit,5,5,dm1,a3);
    t.test_rel_arr(5,g3,h3,1.0e-12,"dtrsv1unn operator[] ");
    t.test_rel_arr(5,v3,h3,1.0e-12,"dtrsv2unn operator[] ");
    t.test_rel_arr(5,a3,h3,1.0e-12,"dtrsv2unn operator[] ");

    // dtrsv, Lower, NoTrans, NonUnit
    cout << "dtrsv, Lower, NoTrans, NonUnit: " << endl;
    gsl_blas_dtrsv(CblasLower,CblasNoTrans,CblasNonUnit,&gm1,&g3);
    cblas_dtrsv(CblasRowMajor,CblasLower,CblasNoTrans,CblasNonUnit,
		5,hm1.data,5,h3.data,1);
    o2scl_cblas::dtrsv(o2scl_cblas::o2cblas_RowMajor,
		       o2scl_cblas::o2cblas_Lower,
		       o2scl_cblas::o2cblas_NoTrans,
		       o2scl_cblas::o2cblas_NonUnit,5,5,m1,v3);
    o2scl_cblas::dtrsv(o2scl_cblas::o2cblas_RowMajor,
		       o2scl_cblas::o2cblas_Lower,
		       o2scl_cblas::o2cblas_NoTrans,
		       o2scl_cblas::o2cblas_NonUnit,5,5,dm1,a3);
    t.test_rel_arr(5,g3,h3,1.0e-12,"dtrsv1lnn operator[] ");
    t.test_rel_arr(5,v3,h3,1.0e-12,"dtrsv2lnn operator[] ");
    t.test_rel_arr(5,a3,h3,1.0e-12,"dtrsv2lnn operator[] ");

    // dtrsv, Upper, Trans, NonUnit
    cout << "dtrsv, Upper, Trans, NonUnit: " << endl;
    gsl_blas_dtrsv(CblasUpper,CblasTrans,CblasNonUnit,&gm1,&g3);
    cblas_dtrsv(CblasRowMajor,CblasUpper,CblasTrans,CblasNonUnit,
		5,hm1.data,5,h3.data,1);
    o2scl_cblas::dtrsv(o2scl_cblas::o2cblas_RowMajor,
		       o2scl_cblas::o2cblas_Upper,
		       o2scl_cblas::o2cblas_Trans,
		       o2scl_cblas::o2cblas_NonUnit,5,5,m1,v3);
    o2scl_cblas::dtrsv(o2scl_cblas::o2cblas_RowMajor,
		       o2scl_cblas::o2cblas_Upper,
		       o2scl_cblas::o2cblas_Trans,
		       o2scl_cblas::o2cblas_NonUnit,5,5,dm1,a3);
    t.test_rel_arr(5,g3,h3,1.0e-12,"dtrsv1utn operator[] ");
    t.test_rel_arr(5,v3,h3,1.0e-12,"dtrsv2utn operator[] ");
    t.test_rel_arr(5,a3,h3,1.0e-12,"dtrsv2utn operator[] ");

    // dtrsv, Lower, Trans, NonUnit
    cout << "dtrsv, Lower, Trans, NonUnit: " << endl;
    gsl_blas_dtrsv(CblasLower,CblasTrans,CblasNonUnit,&gm1,&g3);
    cblas_dtrsv(CblasRowMajor,CblasLower,CblasTrans,CblasNonUnit,
		5,hm1.data,5,h3.data,1);
    o2scl_cblas::dtrsv(o2scl_cblas::o2cblas_RowMajor,
		       o2scl_cblas::o2cblas_Lower,
		       o2scl_cblas::o2cblas_Trans,
		       o2scl_cblas::o2cblas_NonUnit,5,5,m1,v3);
    o2scl_cblas::dtrsv(o2scl_cblas::o2cblas_RowMajor,
		       o2scl_cblas::o2cblas_Lower,
		       o2scl_cblas::o2cblas_Trans,
		       o2scl_cblas::o2cblas_NonUnit,5,5,dm1,a3);
    t.test_rel_arr(5,g3,h3,1.0e-12,"dtrsv1ltn operator[] ");
    t.test_rel_arr(5,v3,h3,1.0e-12,"dtrsv2ltn operator[] ");
    t.test_rel_arr(5,a3,h3,1.0e-12,"dtrsv2ltn operator[] ");

    // dgemm, NoTrans, NoTrans
    cout << "dgemm, NoTrans, NoTrans: " << endl;
    gsl_blas_dgemm(CblasNoTrans,CblasNoTrans,0.1,&gm1,&gm2,0.2,&gm3);
    cblas_dgemm(CblasRowMajor,CblasNoTrans,CblasNoTrans,5,5,5,0.1,hm1.data,5,
		hm2.data,5,0.2,hm3.data,5);
    o2scl_cblas::dgemm(o2scl_cblas::o2cblas_RowMajor,
		       o2scl_cblas::o2cblas_NoTrans,
		       o2scl_cblas::o2cblas_NoTrans,
		       5,5,5,0.1,m1,m2,0.2,m3);
    o2scl_cblas::dgemm(o2scl_cblas::o2cblas_RowMajor,
		       o2scl_cblas::o2cblas_NoTrans,
		       o2scl_cblas::o2cblas_NoTrans,
		       5,5,5,0.1,dm1,dm2,0.2,dm3);
    t.test_rel_mat(5,5,gm3,hm3,1.0e-12,"dgemm1ntnt operator[] ");
    t.test_rel_mat(5,5,m3,hm3,1.0e-12,"dgemm2ntnt operator[] ");
    t.test_rel_mat(5,5,dm3,hm3,1.0e-12,"dgemm3ntnt operator[] ");
    
    // dgemm, Trans, NoTrans
    cout << "dgemm, Trans, NoTrans: " << endl;
    gsl_blas_dgemm(CblasTrans,CblasNoTrans,0.1,&gm1,&gm2,0.2,&gm3);
    cblas_dgemm(CblasRowMajor,CblasTrans,CblasNoTrans,5,5,5,0.1,hm1.data,5,
		hm2.data,5,0.2,hm3.data,5);
    o2scl_cblas::dgemm(o2scl_cblas::o2cblas_RowMajor,
		       o2scl_cblas::o2cblas_Trans,
		       o2scl_cblas::o2cblas_NoTrans,
		       5,5,5,0.1,m1,m2,0.2,m3);
    o2scl_cblas::dgemm(o2scl_cblas::o2cblas_RowMajor,
		       o2scl_cblas::o2cblas_Trans,
		       o2scl_cblas::o2cblas_NoTrans,
		       5,5,5,0.1,dm1,dm2,0.2,dm3);
    t.test_rel_mat(5,5,gm3,hm3,1.0e-12,"dgemm1tnt operator[] ");
    t.test_rel_mat(5,5,m3,hm3,1.0e-12,"dgemm2tnt operator[] ");
    t.test_rel_mat(5,5,dm3,hm3,1.0e-12,"dgemm3tnt operator[] ");

    // dgemm, NoTrans, Trans
    cout << "dgemm, NoTrans, Trans: " << endl;
    gsl_blas_dgemm(CblasNoTrans,CblasTrans,0.1,&gm1,&gm2,0.2,&gm3);
    cblas_dgemm(CblasRowMajor,CblasNoTrans,CblasTrans,5,5,5,0.1,hm1.data,5,
		hm2.data,5,0.2,hm3.data,5);
    o2scl_cblas::dgemm(o2scl_cblas::o2cblas_RowMajor,
		       o2scl_cblas::o2cblas_NoTrans,
		       o2scl_cblas::o2cblas_Trans,
		       5,5,5,0.1,m1,m2,0.2,m3);
    o2scl_cblas::dgemm(o2scl_cblas::o2cblas_RowMajor,
		       o2scl_cblas::o2cblas_NoTrans,
		       o2scl_cblas::o2cblas_Trans,
		       5,5,5,0.1,dm1,dm2,0.2,dm3);
    t.test_rel_mat(5,5,gm3,hm3,1.0e-12,"dgemm1ntt operator[] ");
    t.test_rel_mat(5,5,m3,hm3,1.0e-12,"dgemm2ntt operator[] ");
    t.test_rel_mat(5,5,dm3,hm3,1.0e-12,"dgemm3ntt operator[] ");

    // dgemm, Trans, Trans
    cout << "dgemm, Trans, Trans: " << endl;
    gsl_blas_dgemm(CblasTrans,CblasTrans,0.1,&gm1,&gm2,0.2,&gm3);
    cblas_dgemm(CblasRowMajor,CblasTrans,CblasTrans,5,5,5,0.1,hm1.data,5,
		hm2.data,5,0.2,hm3.data,5);
    o2scl_cblas::dgemm(o2scl_cblas::o2cblas_RowMajor,
		       o2scl_cblas::o2cblas_Trans,
		       o2scl_cblas::o2cblas_Trans,
		       5,5,5,0.1,m1,m2,0.2,m3);
    o2scl_cblas::dgemm(o2scl_cblas::o2cblas_RowMajor,
		       o2scl_cblas::o2cblas_Trans,
		       o2scl_cblas::o2cblas_Trans,
		       5,5,5,0.1,dm1,dm2,0.2,dm3);
    t.test_rel_mat(5,5,gm3,hm3,1.0e-12,"dgemm1tt operator[] ");
    t.test_rel_mat(5,5,m3,hm3,1.0e-12,"dgemm2tt operator[] ");
    t.test_rel_mat(5,5,dm3,hm3,1.0e-12,"dgemm3tt operator[] ");

    // dgemm, ColMajor, NoTrans, NoTrans
    cout << "dgemm, ColMajor, NoTrans, NoTrans: " << endl;
    cblas_dgemm(CblasColMajor,CblasNoTrans,CblasNoTrans,5,5,5,0.1,hm1.data,5,
		hm2.data,5,0.2,hm3.data,5);
    o2scl_cblas::dgemm(o2scl_cblas::o2cblas_ColMajor,
		       o2scl_cblas::o2cblas_NoTrans,
		       o2scl_cblas::o2cblas_NoTrans,
		       5,5,5,0.1,m1,m2,0.2,m3);
    o2scl_cblas::dgemm(o2scl_cblas::o2cblas_ColMajor,
		       o2scl_cblas::o2cblas_NoTrans,
		       o2scl_cblas::o2cblas_NoTrans,
		       5,5,5,0.1,dm1,dm2,0.2,dm3);
    t.test_rel_mat(5,5,m3,hm3,1.0e-12,"dgemmc2ntnt operator[] ");
    t.test_rel_mat(5,5,dm3,hm3,1.0e-12,"dgemmc3ntnt operator[] ");
    
    // dgemm, ColMajor, Trans, NoTrans
    cout << "dgemm, ColMajor, Trans, NoTrans: " << endl;
    cblas_dgemm(CblasColMajor,CblasTrans,CblasNoTrans,5,5,5,0.1,hm1.data,5,
		hm2.data,5,0.2,hm3.data,5);
    o2scl_cblas::dgemm(o2scl_cblas::o2cblas_ColMajor,
		       o2scl_cblas::o2cblas_Trans,
		       o2scl_cblas::o2cblas_NoTrans,
		       5,5,5,0.1,m1,m2,0.2,m3);
    o2scl_cblas::dgemm(o2scl_cblas::o2cblas_ColMajor,
		       o2scl_cblas::o2cblas_Trans,
		       o2scl_cblas::o2cblas_NoTrans,
		       5,5,5,0.1,dm1,dm2,0.2,dm3);
    t.test_rel_mat(5,5,m3,hm3,1.0e-12,"dgemmc2tnt operator[] ");
    t.test_rel_mat(5,5,dm3,hm3,1.0e-12,"dgemmc3tnt operator[] ");

    // dgemm, ColMajor, NoTrans, Trans
    cout << "dgemm, ColMajor, NoTrans, Trans: " << endl;
    cblas_dgemm(CblasColMajor,CblasNoTrans,CblasTrans,5,5,5,0.1,hm1.data,5,
		hm2.data,5,0.2,hm3.data,5);
    o2scl_cblas::dgemm(o2scl_cblas::o2cblas_ColMajor,
		       o2scl_cblas::o2cblas_NoTrans,
		       o2scl_cblas::o2cblas_Trans,
		       5,5,5,0.1,m1,m2,0.2,m3);
    o2scl_cblas::dgemm(o2scl_cblas::o2cblas_ColMajor,
		       o2scl_cblas::o2cblas_NoTrans,
		       o2scl_cblas::o2cblas_Trans,
		       5,5,5,0.1,dm1,dm2,0.2,dm3);
    t.test_rel_mat(5,5,m3,hm3,1.0e-12,"dgemmc2ntt operator[] ");
    t.test_rel_mat(5,5,dm3,hm3,1.0e-12,"dgemmc3ntt operator[] ");

    // dgemm, ColMajor, Trans, Trans
    cout << "dgemm, ColMajor, Trans, Trans: " << endl;
    cblas_dgemm(CblasColMajor,CblasTrans,CblasTrans,5,5,5,0.1,hm1.data,5,
		hm2.data,5,0.2,hm3.data,5);
    o2scl_cblas::dgemm(o2scl_cblas::o2cblas_ColMajor,
		       o2scl_cblas::o2cblas_Trans,
		       o2scl_cblas::o2cblas_Trans,
		       5,5,5,0.1,m1,m2,0.2,m3);
    o2scl_cblas::dgemm(o2scl_cblas::o2cblas_ColMajor,
		       o2scl_cblas::o2cblas_Trans,
		       o2scl_cblas::o2cblas_Trans,
		       5,5,5,0.1,dm1,dm2,0.2,dm3);
    t.test_rel_mat(5,5,m3,hm3,1.0e-12,"dgemmc2tt operator[] ");
    t.test_rel_mat(5,5,dm3,hm3,1.0e-12,"dgemmc3tt operator[] ");

    // daxpy_subvec
    cout << "daxpy_subvec: " << endl;
    for(size_t i=0;i<5;i++) {
      v1[i]=sin((double)(i+1));
      v2[i]=cos((double)(i+1));
      g1[i]=sin((double)(i+1));
      g2[i]=cos((double)(i+1));
    }
    for(size_t k=0;k<5;k++) {
      ovector_subvector vsub1(v1,k,5-k);
      ovector_subvector vsub2(v2,k,5-k);
      o2scl_cblas::daxpy(0.5,5-k,vsub1,vsub2);
      o2scl_cblas::daxpy_subvec(0.5,5,g1,g2,k);
      t.test_rel_arr(5,v2,g2,1.0e-12,"daxpy_subvec1 operator[] ");
    }

    // ddot_subvec
    cout << "ddot_subvec: " << endl;
    for(size_t k=0;k<5;k++) {
      ovector_subvector vsub1(v1,k,5-k);
      ovector_subvector vsub2(v2,k,5-k);
      double tx1=o2scl_cblas::ddot(5-k,vsub1,vsub2);
      double tx2=o2scl_cblas::ddot_subvec(5,g1,g2,k);
      t.test_rel(tx1,tx2,1.0e-12,"ddot_subvec1 operator[] ");
    }

    // dscal_subvec
    cout << "dscal_subvec: " << endl;
    for(size_t k=0;k<5;k++) {
      ovector_subvector vsub1(v1,k,5-k);
      o2scl_cblas::dscal(0.5,5-k,vsub1);
      o2scl_cblas::dscal_subvec(0.5,5,g1,k);
    }
  
    // dnrm2_subvec
    cout << "dnrm2_subvec: " << endl;
    for(size_t k=0;k<5;k++) {
      ovector_subvector vsub1(v1,k,5-k);
      double tx1=o2scl_cblas::dnrm2(5-k,vsub1);
      double tx2=o2scl_cblas::dnrm2_subvec(5,g1,k);
      t.test_rel(tx1,tx2,1.0e-12,"dnrm2_subvec1 operator[] ");
    }

    // dnrm2_subcol
    cout << "dnrm2_subcol: " << endl;
    for(size_t i=0;i<5;i++) {
      for(size_t j=0;j<5;j++) {
	m1[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
      }
    }
    for(size_t k=0;k<5;k++) {
      for(size_t ell=0;ell<5;ell++) {
	omatrix_col mcol1(m1,k);
	ovector_subvector vsub1(mcol1,ell,5-ell);
	double tx1=o2scl_cblas::dnrm2(5-ell,vsub1);
	double tx2=o2scl_cblas::dnrm2_subcol(m1,ell,k,5);
	t.test_rel(tx1,tx2,1.0e-12,"dnrm2_subcol operator[] ");
      }
    }

    // dscal_subcol
    cout << "dscal_subcol: " << endl;
    for(size_t i=0;i<5;i++) {
      for(size_t j=0;j<5;j++) {
	m1[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
      }
    }
    for(size_t k=0;k<5;k++) {
      for(size_t ell=0;ell<5;ell++) {
	omatrix_col mcol1(m1,k);
	ovector_subvector vsub1(mcol1,ell,5-ell);
	o2scl_cblas::dscal(0.5,5-ell,vsub1);
	o2scl_cblas::dscal_subcol(m1,ell,k,5,0.5);
	for(size_t ij=0;ij<5-ell;ij++) {
	  t.test_rel(vsub1[ij],m1[ij+ell][k],1.0e-12,
		     "dscal_subcol operator[] ");
	}
      }
    }

    // daxpy_subcol
    cout << "daxpy_subcol: " << endl;
    for(size_t i=0;i<5;i++) {
      v1[i]=sin((double)(i+1));
      for(size_t j=0;j<5;j++) {
	m1[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
      }
    }
    for(size_t k=0;k<5;k++) {
      for(size_t ell=0;ell<5;ell++) {
	omatrix_col mcol1(m1,k);
	ovector_subvector vsub1(mcol1,ell,5-ell);
	o2scl_cblas::daxpy(0.5,5-ell,vsub1,v1);
	o2scl_cblas::daxpy_subcol(0.5,5,m1,ell,k,v1);
	for(size_t ij=0;ij<5-ell;ij++) {
	  t.test_rel(vsub1[ij],m1[ij+ell][k],1.0e-12,
		     "daxpy_subcol operator[] ");
	}
      }
    }

    // ddot_subcol
    cout << "ddot_subcol: " << endl;
    for(size_t i=0;i<5;i++) {
      v1[i]=sin((double)(i+1));
      for(size_t j=0;j<5;j++) {
	m1[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
      }
    }
    for(size_t k=0;k<5;k++) {
      for(size_t ell=0;ell<5;ell++) {
	omatrix_col mcol1(m1,k);
	ovector_subvector vsub1(mcol1,ell,5-ell);
	o2scl_cblas::ddot(5-ell,vsub1,v1);
	o2scl_cblas::ddot_subcol(5,m1,ell,k,v1);
	for(size_t ij=0;ij<5-ell;ij++) {
	  t.test_rel(vsub1[ij],m1[ij+ell][k],1.0e-12,
		     "ddot_subcol operator[] ");
	}
      }
    }

    // dnrm2_subrow
    cout << "dnrm2_subrow: " << endl;
    for(size_t i=0;i<5;i++) {
      for(size_t j=0;j<5;j++) {
	m1[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
      }
    }
    for(size_t k=0;k<5;k++) {
      for(size_t ell=0;ell<5;ell++) {
	omatrix_row mrow1(m1,k);
	ovector_subvector vsub1(mrow1,ell,5-ell);
	double tx1=o2scl_cblas::dnrm2(5-ell,vsub1);
	double tx2=o2scl_cblas::dnrm2_subrow(m1,k,ell,5);
	t.test_rel(tx1,tx2,1.0e-12,"dnrm2_subrow operator[] ");
      }
    }

    // dscal_subrow
    cout << "dscal_subrow: " << endl;
    for(size_t i=0;i<5;i++) {
      for(size_t j=0;j<5;j++) {
	m1[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
      }
    }
    for(size_t k=0;k<5;k++) {
      for(size_t ell=0;ell<5;ell++) {
	omatrix_row mrow1(m1,k);
	ovector_subvector vsub1(mrow1,ell,5-ell);
	o2scl_cblas::dscal(0.5,5-ell,vsub1);
	o2scl_cblas::dscal_subrow(m1,k,ell,5,0.5);
	for(size_t ij=0;ij<5-ell;ij++) {
	  t.test_rel(vsub1[ij],m1[k][ij+ell],1.0e-12,
		     "dscal_subrow operator[] ");
	}
      }
    }

    // daxpy_subrow
    cout << "daxpy_subrow: " << endl;
    for(size_t i=0;i<5;i++) {
      v1[i]=sin((double)(i+1));
      for(size_t j=0;j<5;j++) {
	m1[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
      }
    }
    for(size_t k=0;k<5;k++) {
      for(size_t ell=0;ell<5;ell++) {
	omatrix_row mcol1(m1,k);
	ovector_subvector vsub1(mcol1,ell,5-ell);
	o2scl_cblas::daxpy(0.5,5-ell,vsub1,v1);
	o2scl_cblas::daxpy_subrow(0.5,5,m1,k,ell,v1);
	for(size_t ij=0;ij<5-ell;ij++) {
	  t.test_rel(vsub1[ij],m1[k][ij+ell],1.0e-12,
		     "daxpy_subrow operator[] ");
	}
      }
    }

    // ddot_subrow
    cout << "ddot_subrow: " << endl;
    for(size_t i=0;i<5;i++) {
      v1[i]=sin((double)(i+1));
      for(size_t j=0;j<5;j++) {
	m1[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
      }
    }
    for(size_t k=0;k<5;k++) {
      for(size_t ell=0;ell<5;ell++) {
	omatrix_row mcol1(m1,k);
	ovector_subvector vsub1(mcol1,ell,5-ell);
	o2scl_cblas::ddot(5-ell,vsub1,v1);
	o2scl_cblas::ddot_subrow(5,m1,k,ell,v1);
	for(size_t ij=0;ij<5-ell;ij++) {
	  t.test_rel(vsub1[ij],m1[k][ij+ell],1.0e-12,
		     "ddot_subrow operator[] ");
	}
      }
    }

    cout << endl;
  }

  // Test with operator()
  {
    cout << "Test with operator(): " << endl;

    for(size_t i=0;i<5;i++) {
      a1[i]=sin((double)(i+1));
      a2[i]=cos((double)(i+1));
      a3[i]=tan((double)(i+1));
      v1[i]=sin((double)(i+1));
      v2[i]=cos((double)(i+1));
      v3[i]=tan((double)(i+1));
      g1[i]=sin((double)(i+1));
      g2[i]=cos((double)(i+1));
      g3[i]=tan((double)(i+1));
      h1[i]=sin((double)(i+1));
      h2[i]=cos((double)(i+1));
      h3[i]=tan((double)(i+1));
      for(size_t j=0;j<5;j++) {
	dm1[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
	m1[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
	gm1[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
	hm1[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
	dm2[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
	m2[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
	gm2[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
	hm2[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
	dm3[i][j]=0.0;
	m3[i][j]=0.0;
	gm3[i][j]=0.0;
	hm3[i][j]=0.0;
      }
    }
  
    // daxpy
    cout << "daxpy: " << endl;
    gsl_blas_daxpy(0.5,&g1,&g2);
    cblas_daxpy(5,0.5,h1.data,1,h2.data,1);
    o2scl_cblas_paren::daxpy(0.5,5,v1,v2);
    t.test_rel_arr(5,g2,h2,1.0e-12,"daxpy1 operator() ");
    t.test_rel_arr(5,v2,h2,1.0e-12,"daxpy2 operator() ");

    // ddot
    cout << "ddot: " << endl;
    gsl_blas_ddot(&g1,&g2,&res1);
    res2=cblas_ddot(5,h1.data,1,h2.data,1);
    res3=o2scl_cblas_paren::ddot(5,v1,v2);
    t.test_rel(res1,res2,1.0e-12,"ddot1 operator() ");
    t.test_rel(res1,res3,1.0e-12,"ddot2 operator() ");
  
    // dnrm2
    cout << "dnrm2: " << endl;
    res1=gsl_blas_dnrm2(&g1);
    res2=cblas_dnrm2(5,h1.data,1);
    res3=o2scl_cblas_paren::dnrm2(5,v1);
    t.test_rel(res1,res2,1.0e-12,"dnrm21 operator() ");
    t.test_rel(res1,res3,1.0e-12,"dnrm22 operator() ");
  
    // dscal
    cout << "dscal: " << endl;
    gsl_blas_dscal(0.5,&g3);
    cblas_dscal(5,0.5,h3.data,1);
    o2scl_cblas_paren::dscal(0.5,5,v3);
    t.test_rel_arr(5,g3,h3,1.0e-12,"dscal1 operator() ");
    t.test_rel_arr(5,v3,h3,1.0e-12,"dscal2 operator() ");
  
    // dgemv, NoTrans
    cout << "dgemv, NoTrans: " << endl;
    gsl_blas_dgemv(CblasNoTrans,0.1,&gm1,&g1,0.2,&g2);
    cblas_dgemv(CblasRowMajor,CblasNoTrans,5,5,0.1,hm1.data,5,
		h1.data,1,0.2,h2.data,1);
    o2scl_cblas_paren::dgemv(o2scl_cblas_paren::o2cblas_RowMajor,
			     o2scl_cblas_paren::o2cblas_NoTrans,
			     5,5,0.1,m1,v1,0.2,v2);
    t.test_rel_arr(5,g2,h2,1.0e-12,"dgemv1 operator() ");
    t.test_rel_arr(5,v2,h2,1.0e-12,"dgemv2 operator() ");

    // dgemv, Trans (Note that ConjTrans was not originally supported)
    cout << "dgemv, Trans: " << endl;
    gsl_blas_dgemv(CblasTrans,0.1,&gm1,&g1,0.2,&g2);
    cblas_dgemv(CblasRowMajor,CblasTrans,5,5,0.1,hm1.data,5,
		h1.data,1,0.2,h2.data,1);
    o2scl_cblas_paren::dgemv(o2scl_cblas_paren::o2cblas_RowMajor,
			     o2scl_cblas_paren::o2cblas_Trans,
			     5,5,0.1,m1,v1,0.2,v2);
    t.test_rel_arr(5,g2,h2,1.0e-12,"dgemv1t operator() ");
    t.test_rel_arr(5,v2,h2,1.0e-12,"dgemv2t operator() ");

    // dtrsv, Upper, NoTrans, NonUnit
    cout << "dtrsv, Upper, NoTrans, NonUnit: " << endl;
    gsl_blas_dtrsv(CblasUpper,CblasNoTrans,CblasNonUnit,&gm1,&g3);
    cblas_dtrsv(CblasRowMajor,CblasUpper,CblasNoTrans,CblasNonUnit,
		5,hm1.data,5,h3.data,1);
    o2scl_cblas_paren::dtrsv(o2scl_cblas_paren::o2cblas_RowMajor,
			     o2scl_cblas_paren::o2cblas_Upper,
			     o2scl_cblas_paren::o2cblas_NoTrans,
			     o2scl_cblas_paren::o2cblas_NonUnit,
			     5,5,m1,v3);
    t.test_rel_arr(5,g3,h3,1.0e-12,"dtrsv1unn operator() ");
    t.test_rel_arr(5,v3,h3,1.0e-12,"dtrsv2unn operator() ");

    // dtrsv, Lower, NoTrans, NonUnit
    cout << "dtrsv, Lower, NoTrans, NonUnit: " << endl;
    gsl_blas_dtrsv(CblasLower,CblasNoTrans,CblasNonUnit,&gm1,&g3);
    cblas_dtrsv(CblasRowMajor,CblasLower,CblasNoTrans,CblasNonUnit,
		5,hm1.data,5,h3.data,1);
    o2scl_cblas_paren::dtrsv(o2scl_cblas_paren::o2cblas_RowMajor,
			     o2scl_cblas_paren::o2cblas_Lower,
			     o2scl_cblas_paren::o2cblas_NoTrans,
			     o2scl_cblas_paren::o2cblas_NonUnit,
			     5,5,m1,v3);
    t.test_rel_arr(5,g3,h3,1.0e-12,"dtrsv1lnn operator() ");
    t.test_rel_arr(5,v3,h3,1.0e-12,"dtrsv2lnn operator() ");

    // dtrsv, Upper, Trans, NonUnit
    cout << "dtrsv, Upper, Trans, NonUnit: " << endl;
    gsl_blas_dtrsv(CblasUpper,CblasTrans,CblasNonUnit,&gm1,&g3);
    cblas_dtrsv(CblasRowMajor,CblasUpper,CblasTrans,CblasNonUnit,
		5,hm1.data,5,h3.data,1);
    o2scl_cblas_paren::dtrsv(o2scl_cblas_paren::o2cblas_RowMajor,
			     o2scl_cblas_paren::o2cblas_Upper,
			     o2scl_cblas_paren::o2cblas_Trans,
			     o2scl_cblas_paren::o2cblas_NonUnit,
			     5,5,m1,v3);
    t.test_rel_arr(5,g3,h3,1.0e-12,"dtrsv1utn operator() ");
    t.test_rel_arr(5,v3,h3,1.0e-12,"dtrsv2utn operator() ");

    // dtrsv, Lower, Trans, NonUnit
    cout << "dtrsv, Lower, Trans, NonUnit: " << endl;
    gsl_blas_dtrsv(CblasLower,CblasTrans,CblasNonUnit,&gm1,&g3);
    cblas_dtrsv(CblasRowMajor,CblasLower,CblasTrans,CblasNonUnit,
		5,hm1.data,5,h3.data,1);
    o2scl_cblas_paren::dtrsv(o2scl_cblas_paren::o2cblas_RowMajor,
			     o2scl_cblas_paren::o2cblas_Lower,
			     o2scl_cblas_paren::o2cblas_Trans,
			     o2scl_cblas_paren::o2cblas_NonUnit,
			     5,5,m1,v3);
    t.test_rel_arr(5,g3,h3,1.0e-12,"dtrsv1ltn operator() ");
    t.test_rel_arr(5,v3,h3,1.0e-12,"dtrsv2ltn operator() ");

    // dgemm, NoTrans, NoTrans
    cout << "dgemm, NoTrans, NoTrans: " << endl;
    gsl_blas_dgemm(CblasNoTrans,CblasNoTrans,0.1,&gm1,&gm2,0.2,&gm3);
    cblas_dgemm(CblasRowMajor,CblasNoTrans,CblasNoTrans,5,5,5,0.1,hm1.data,5,
		hm2.data,5,0.2,hm3.data,5);
    o2scl_cblas_paren::dgemm(o2scl_cblas_paren::o2cblas_RowMajor,
			     o2scl_cblas_paren::o2cblas_NoTrans,
			     o2scl_cblas_paren::o2cblas_NoTrans,
			     5,5,5,0.1,m1,m2,0.2,m3);
    t.test_rel_mat(5,5,gm3,hm3,1.0e-12,"paren_dgemm1_ntnt operator() ");
    t.test_rel_mat(5,5,m3,hm3,1.0e-12,"paren_dgemm2_ntnt operator() ");
    
    // dgemm, Trans, NoTrans
    cout << "dgemm, Trans, NoTrans: " << endl;
    gsl_blas_dgemm(CblasTrans,CblasNoTrans,0.1,&gm1,&gm2,0.2,&gm3);
    cblas_dgemm(CblasRowMajor,CblasTrans,CblasNoTrans,5,5,5,0.1,hm1.data,5,
		hm2.data,5,0.2,hm3.data,5);
    o2scl_cblas_paren::dgemm(o2scl_cblas_paren::o2cblas_RowMajor,
		       o2scl_cblas_paren::o2cblas_Trans,
		       o2scl_cblas_paren::o2cblas_NoTrans,
		       5,5,5,0.1,m1,m2,0.2,m3);
    t.test_rel_mat(5,5,gm3,hm3,1.0e-12,"paren_dgemm1tnt operator() ");
    t.test_rel_mat(5,5,m3,hm3,1.0e-12,"paren_dgemm2tnt operator() ");

    // dgemm, NoTrans, Trans
    cout << "dgemm, NoTrans, Trans: " << endl;
    gsl_blas_dgemm(CblasNoTrans,CblasTrans,0.1,&gm1,&gm2,0.2,&gm3);
    cblas_dgemm(CblasRowMajor,CblasNoTrans,CblasTrans,5,5,5,0.1,hm1.data,5,
		hm2.data,5,0.2,hm3.data,5);
    o2scl_cblas_paren::dgemm(o2scl_cblas_paren::o2cblas_RowMajor,
		       o2scl_cblas_paren::o2cblas_NoTrans,
		       o2scl_cblas_paren::o2cblas_Trans,
		       5,5,5,0.1,m1,m2,0.2,m3);
    t.test_rel_mat(5,5,gm3,hm3,1.0e-12,"paren_dgemm1ntt operator() ");
    t.test_rel_mat(5,5,m3,hm3,1.0e-12,"paren_dgemm2ntt operator() ");

    // dgemm, Trans, Trans
    cout << "dgemm, Trans, Trans: " << endl;
    gsl_blas_dgemm(CblasTrans,CblasTrans,0.1,&gm1,&gm2,0.2,&gm3);
    cblas_dgemm(CblasRowMajor,CblasTrans,CblasTrans,5,5,5,0.1,hm1.data,5,
		hm2.data,5,0.2,hm3.data,5);
    o2scl_cblas_paren::dgemm(o2scl_cblas_paren::o2cblas_RowMajor,
		       o2scl_cblas_paren::o2cblas_Trans,
		       o2scl_cblas_paren::o2cblas_Trans,
		       5,5,5,0.1,m1,m2,0.2,m3);
    t.test_rel_mat(5,5,gm3,hm3,1.0e-12,"paren_dgemm1tt operator() ");
    t.test_rel_mat(5,5,m3,hm3,1.0e-12,"paren_dgemm2tt operator() ");

    // dgemm, ColMajor, NoTrans, NoTrans
    cout << "dgemm, ColMajor, NoTrans, NoTrans: " << endl;
    cblas_dgemm(CblasColMajor,CblasNoTrans,CblasNoTrans,5,5,5,0.1,hm1.data,5,
		hm2.data,5,0.2,hm3.data,5);
    o2scl_cblas_paren::dgemm(o2scl_cblas_paren::o2cblas_ColMajor,
		       o2scl_cblas_paren::o2cblas_NoTrans,
		       o2scl_cblas_paren::o2cblas_NoTrans,
		       5,5,5,0.1,m1,m2,0.2,m3);
    t.test_rel_mat(5,5,m3,hm3,1.0e-12,"paren_dgemmc2ntnt operator() ");
    
    // dgemm, ColMajor, Trans, NoTrans
    cout << "dgemm, ColMajor, Trans, NoTrans: " << endl;
    cblas_dgemm(CblasColMajor,CblasTrans,CblasNoTrans,5,5,5,0.1,hm1.data,5,
		hm2.data,5,0.2,hm3.data,5);
    o2scl_cblas_paren::dgemm(o2scl_cblas_paren::o2cblas_ColMajor,
		       o2scl_cblas_paren::o2cblas_Trans,
		       o2scl_cblas_paren::o2cblas_NoTrans,
		       5,5,5,0.1,m1,m2,0.2,m3);
    t.test_rel_mat(5,5,m3,hm3,1.0e-12,"paren_dgemmc2tnt operator() ");

    // dgemm, ColMajor, NoTrans, Trans
    cout << "dgemm, ColMajor, NoTrans, Trans: " << endl;
    cblas_dgemm(CblasColMajor,CblasNoTrans,CblasTrans,5,5,5,0.1,hm1.data,5,
		hm2.data,5,0.2,hm3.data,5);
    o2scl_cblas_paren::dgemm(o2scl_cblas_paren::o2cblas_ColMajor,
		       o2scl_cblas_paren::o2cblas_NoTrans,
		       o2scl_cblas_paren::o2cblas_Trans,
		       5,5,5,0.1,m1,m2,0.2,m3);
    t.test_rel_mat(5,5,m3,hm3,1.0e-12,"paren_dgemmc2ntt operator() ");

    // dgemm, ColMajor, Trans, Trans
    cout << "dgemm, ColMajor, Trans, Trans: " << endl;
    cblas_dgemm(CblasColMajor,CblasTrans,CblasTrans,5,5,5,0.1,hm1.data,5,
		hm2.data,5,0.2,hm3.data,5);
    o2scl_cblas_paren::dgemm(o2scl_cblas_paren::o2cblas_ColMajor,
		       o2scl_cblas_paren::o2cblas_Trans,
		       o2scl_cblas_paren::o2cblas_Trans,
		       5,5,5,0.1,m1,m2,0.2,m3);
    t.test_rel_mat(5,5,m3,hm3,1.0e-12,"paren_dgemmc2tt operator() ");

    // daxpy_subvec
    cout << "daxpy_subvec: " << endl;
    for(size_t i=0;i<5;i++) {
      v1[i]=sin((double)(i+1));
      v2[i]=cos((double)(i+1));
      g1[i]=sin((double)(i+1));
      g2[i]=cos((double)(i+1));
    }
    for(size_t k=0;k<5;k++) {
      ovector_subvector vsub1(v1,k,5-k);
      ovector_subvector vsub2(v2,k,5-k);
      o2scl_cblas_paren::daxpy(0.5,5-k,vsub1,vsub2);
      o2scl_cblas_paren::daxpy_subvec(0.5,5,g1,g2,k);
      t.test_rel_arr(5,v2,g2,1.0e-12,"daxpy_subvec1 operator() ");
    }

    // ddot_subvec
    cout << "ddot_subvec: " << endl;
    for(size_t k=0;k<5;k++) {
      ovector_subvector vsub1(v1,k,5-k);
      ovector_subvector vsub2(v2,k,5-k);
      double tx1=o2scl_cblas_paren::ddot(5-k,vsub1,vsub2);
      double tx2=o2scl_cblas_paren::ddot_subvec(5,g1,g2,k);
      t.test_rel(tx1,tx2,1.0e-12,"ddot_subvec1 operator() ");
    }

    // dscal_subvec
    cout << "dscal_subvec: " << endl;
    for(size_t k=0;k<5;k++) {
      ovector_subvector vsub1(v1,k,5-k);
      o2scl_cblas_paren::dscal(0.5,5-k,vsub1);
      o2scl_cblas_paren::dscal_subvec(0.5,5,g1,k);
    }
  
    // dnrm2_subvec
    cout << "dnrm2_subvec: " << endl;
    for(size_t k=0;k<5;k++) {
      ovector_subvector vsub1(v1,k,5-k);
      double tx1=o2scl_cblas_paren::dnrm2(5-k,vsub1);
      double tx2=o2scl_cblas_paren::dnrm2_subvec(5,g1,k);
      t.test_rel(tx1,tx2,1.0e-12,"dnrm2_subvec1 operator() ");
    }

    // dnrm2_subcol
    cout << "dnrm2_subcol: " << endl;
    for(size_t i=0;i<5;i++) {
      for(size_t j=0;j<5;j++) {
	m1[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
      }
    }
    for(size_t k=0;k<5;k++) {
      for(size_t ell=0;ell<5;ell++) {
	omatrix_col mcol1(m1,k);
	ovector_subvector vsub1(mcol1,ell,5-ell);
	double tx1=o2scl_cblas_paren::dnrm2(5-ell,vsub1);
	double tx2=o2scl_cblas_paren::dnrm2_subcol(m1,ell,k,5);
	t.test_rel(tx1,tx2,1.0e-12,"dnrm2_subcol operator() ");
      }
    }

    // dscal_subcol
    cout << "dscal_subcol: " << endl;
    for(size_t i=0;i<5;i++) {
      for(size_t j=0;j<5;j++) {
	m1[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
      }
    }
    for(size_t k=0;k<5;k++) {
      for(size_t ell=0;ell<5;ell++) {
	omatrix_col mcol1(m1,k);
	ovector_subvector vsub1(mcol1,ell,5-ell);
	o2scl_cblas_paren::dscal(0.5,5-ell,vsub1);
	o2scl_cblas_paren::dscal_subcol(m1,ell,k,5,0.5);
	for(size_t ij=0;ij<5-ell;ij++) {
	  t.test_rel(vsub1[ij],m1[ij+ell][k],1.0e-12,
		     "dscal_subcol operator() ");
	}
      }
    }

    // daxpy_subcol
    cout << "daxpy_subcol: " << endl;
    for(size_t i=0;i<5;i++) {
      v1[i]=sin((double)(i+1));
      for(size_t j=0;j<5;j++) {
	m1[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
      }
    }
    for(size_t k=0;k<5;k++) {
      for(size_t ell=0;ell<5;ell++) {
	omatrix_col mcol1(m1,k);
	ovector_subvector vsub1(mcol1,ell,5-ell);
	o2scl_cblas_paren::daxpy(0.5,5-ell,vsub1,v1);
	o2scl_cblas_paren::daxpy_subcol(0.5,5,m1,ell,k,v1);
	for(size_t ij=0;ij<5-ell;ij++) {
	  t.test_rel(vsub1[ij],m1[ij+ell][k],1.0e-12,
		     "daxpy_subcol operator() ");
	}
      }
    }

    // ddot_subcol
    cout << "ddot_subcol: " << endl;
    for(size_t i=0;i<5;i++) {
      v1[i]=sin((double)(i+1));
      for(size_t j=0;j<5;j++) {
	m1[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
      }
    }
    for(size_t k=0;k<5;k++) {
      for(size_t ell=0;ell<5;ell++) {
	omatrix_col mcol1(m1,k);
	ovector_subvector vsub1(mcol1,ell,5-ell);
	o2scl_cblas_paren::ddot(5-ell,vsub1,v1);
	o2scl_cblas_paren::ddot_subcol(5,m1,ell,k,v1);
	for(size_t ij=0;ij<5-ell;ij++) {
	  t.test_rel(vsub1[ij],m1[ij+ell][k],1.0e-12,
		     "ddot_subcol operator() ");
	}
      }
    }

    // dnrm2_subrow
    cout << "dnrm2_subrow: " << endl;
    for(size_t i=0;i<5;i++) {
      for(size_t j=0;j<5;j++) {
	m1[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
      }
    }
    for(size_t k=0;k<5;k++) {
      for(size_t ell=0;ell<5;ell++) {
	omatrix_row mrow1(m1,k);
	ovector_subvector vsub1(mrow1,ell,5-ell);
	double tx1=o2scl_cblas_paren::dnrm2(5-ell,vsub1);
	double tx2=o2scl_cblas_paren::dnrm2_subrow(m1,k,ell,5);
	t.test_rel(tx1,tx2,1.0e-12,"dnrm2_subrow operator() ");
      }
    }

    // dscal_subrow
    cout << "dscal_subrow: " << endl;
    for(size_t i=0;i<5;i++) {
      for(size_t j=0;j<5;j++) {
	m1[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
      }
    }
    for(size_t k=0;k<5;k++) {
      for(size_t ell=0;ell<5;ell++) {
	omatrix_row mrow1(m1,k);
	ovector_subvector vsub1(mrow1,ell,5-ell);
	o2scl_cblas_paren::dscal(0.5,5-ell,vsub1);
	o2scl_cblas_paren::dscal_subrow(m1,k,ell,5,0.5);
	for(size_t ij=0;ij<5-ell;ij++) {
	  t.test_rel(vsub1[ij],m1[k][ij+ell],1.0e-12,
		     "dscal_subrow operator() ");
	}
      }
    }

    // daxpy_subrow
    cout << "daxpy_subrow: " << endl;
    for(size_t i=0;i<5;i++) {
      v1[i]=sin((double)(i+1));
      for(size_t j=0;j<5;j++) {
	m1[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
      }
    }
    for(size_t k=0;k<5;k++) {
      for(size_t ell=0;ell<5;ell++) {
	omatrix_row mcol1(m1,k);
	ovector_subvector vsub1(mcol1,ell,5-ell);
	o2scl_cblas_paren::daxpy(0.5,5-ell,vsub1,v1);
	o2scl_cblas_paren::daxpy_subrow(0.5,5,m1,k,ell,v1);
	for(size_t ij=0;ij<5-ell;ij++) {
	  t.test_rel(vsub1[ij],m1[k][ij+ell],1.0e-12,
		     "daxpy_subrow operator() ");
	}
      }
    }

    // ddot_subrow
    cout << "ddot_subrow: " << endl;
    for(size_t i=0;i<5;i++) {
      v1[i]=sin((double)(i+1));
      for(size_t j=0;j<5;j++) {
	m1[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
      }
    }
    for(size_t k=0;k<5;k++) {
      for(size_t ell=0;ell<5;ell++) {
	omatrix_row mcol1(m1,k);
	ovector_subvector vsub1(mcol1,ell,5-ell);
	o2scl_cblas_paren::ddot(5-ell,vsub1,v1);
	o2scl_cblas_paren::ddot_subrow(5,m1,k,ell,v1);
	for(size_t ij=0;ij<5-ell;ij++) {
	  t.test_rel(vsub1[ij],m1[k][ij+ell],1.0e-12,
		     "ddot_subrow operator() ");
	}
      }
    }

    cout << endl;
  }

  t.report();
  return 0;
}

