/*
  -------------------------------------------------------------------

  Copyright (C) 2006-2012, Andrew W. Steiner

  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifndef O2SCL_HDF_NUCMASS_IO_H
#define O2SCL_HDF_NUCMASS_IO_H

/** \file hdf_nucmass_io.h
    \brief File for HDF input of the \o2 \ref ame_mass and 
    \ref mnmsk_mass data files
*/

#include <hdf5.h>
#include <hdf5_hl.h>

#include <o2scl/constants.h>
#include <o2scl/hdf_file.h>
#include <o2scl/lib_settings.h>
#include <o2scl/nuclear_mass.h>

#ifndef DOXYGENP
namespace o2scl_hdf {
#endif

#ifdef DOXYGENP

  // Separately document these functions since they're in the hidden
  // o2scl namespace
  
  /// Read data for \ref ame_mass from an HDF table
  int ame_load(ame_mass &ame, std::string version, string dir="");

  /// Read data for \ref mnmsk_mass from an HDF table
  int mnmsk_load(mnmsk_mass &mnmsk, string dir="");
  
  /** \brief Read data for \ref hfb_mass from an HDF table
      
      Valid values of \c model at present are 2, 8, and 14, corresponding
      to the HFB2 (\ref Goriely02), HFB8 (\ref Samyn04), and HFB14 
      (\ref Goriely07). If a number other than these three is given,
      the error handler is called. 
  */
  int hfb_load(hfb_mass &hfb, int model=14, string dir="");

  /** \brief Read data for \ref hfb_mass from an HDF table
      
      Valid values of \c model at present are 17 and 21, corresponding
      to the HFB17 (\ref Goriely02) and HFB21 (\ref Samyn04). If a
      number other than these two is given, the error handler is
      called.
  */
  int hfb_sp_load(hfb_sp_mass &hfb, int model=21, string dir="");

#endif

  int ame_load(o2scl::ame_mass &ame, std::string version, 
	       std::string dir="");
  
  int mnmsk_load(o2scl::mnmsk_mass &mnmsk, std::string dir="");
  
  int hfb_load(o2scl::hfb_mass &hfb, int model=14, std::string dir="");

  int hfb_sp_load(o2scl::hfb_sp_mass &hfb, int model=21, std::string dir="");
  
#ifndef DOXYGENP
}
#endif

#endif
