/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006-2012, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <o2scl/mass_fit.h>

using namespace std;
using namespace o2scl;
using namespace o2scl_const;

mass_fit::mass_fit() {
  exp=&def_dist;
  mm=&def_mmin;
  def_mmin.ntrial*=10;
  even_even=false;
  minZ=8;
  minN=8;
}

int mass_fit::set_exp_mass(nuclear_mass &nm) {
  def_dist.set_dist(nm);
  return 0;
}

double mass_fit::min_fun(size_t nv, const ovector_base &x) {

  double y=0.0;
  
  nmf->fit_fun(nv,x);
  
  eval(*nmf,y);
  
  return y;
}

int mass_fit::fit(nuclear_mass_fit &n, double &fmin) {
  
  nmf=&n;
  size_t nv=nmf->nfit;
  ovector mx(nv);
  nmf->guess_fun(nv,mx);
  
  multi_funct_mfptr<mass_fit> mfm(this,&mass_fit::min_fun);
  
  int ret=mm->mmin(nv,mx,fmin,mfm);
  fmin=mfm(nv,mx);
  
  return ret;
}

int mass_fit::eval(nuclear_mass &n, double &fmin) {

  fmin=0.0;

  size_t nn=0;
  for(nuclear_dist::iterator ndi=exp->begin();ndi!=exp->end();ndi++) {
    int Z=ndi->Z;
    int N=ndi->N;
    if (N>=minN && Z>=minZ && (even_even==false || (N%2==0 && Z%2==0))) {
      fmin+=pow(ndi->mex*hc_mev_fm-n.mass_excess(Z,N),2.0);
      nn++;
    }
  }
  fmin=sqrt(fmin/nn);

  // If the result is not finite, just return an arbitrary large value
  if (!gsl_finite(fmin)) {
    fmin=1.0e4;
  }

  return 0;
}

