/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006-2012, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <iostream>
#include <o2scl/test_mgr.h>
#include <o2scl/mass_fit.h>
#include <o2scl/hdf_nucmass_io.h>

using namespace std;
using namespace o2scl;
using namespace o2scl_const;
using namespace o2scl_fm;
using namespace o2scl_hdf;

int main(void) {
  test_mgr t;
  t.set_output_level(1);
  
  cout.setf(ios::scientific);

  double res;
  semi_empirical_mass sem;
  mass_fit mf;

  ame_mass ame;
  o2scl_hdf::ame_load(ame,"");
  ame_mass_exp amex;
  o2scl_hdf::ame_load(amex,"");
  mnmsk_mass_exp mexp;
  o2scl_hdf::mnmsk_load(mexp);
  mnmsk_mass mm;
  o2scl_hdf::mnmsk_load(mm);

  mf.set_exp_mass(ame);
  mf.fit(sem,res);
  cout << sem.B << " " << sem.Sv << " " << sem.Ss << " " 
       << sem.Ec << " " << sem.Epair << endl;
  cout << res << endl;
  t.test_gen(res<4.0,"Successful fit.");

  mf.set_exp_mass(mexp);
  mf.eval(mm,res);
  cout << res << endl;
  t.test_rel(res,0.6806,1.0e-4,"Moller fit 1");

  mf.set_exp_mass(amex);
  mf.eval(mm,res);
  cout << res << endl;
  t.test_rel(res,0.6555,1.0e-4,"Moller fit 2");

  mf.set_exp_mass(ame);
  mf.eval(mm,res);
  cout << res << endl;
  t.test_rel(res,0.8909,1.0e-4,"Moller fit 3");

  t.report();
  return 0;
}
  
