/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006-2012, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/constants.h>
#include <o2scl/nuclear_dist.h>
#include <o2scl/hdf_nucmass_io.h>
#include <o2scl/test_mgr.h>

using namespace std;
using namespace o2scl;
using namespace o2scl_const;
using namespace o2scl_hdf;

int main(void) {
  test_mgr t;
  t.set_output_level(2);
   
  cout.setf(ios::scientific);

  // Load several mass formulae to make distributions out of
  
  ame_mass ame;
  o2scl_hdf::ame_load(ame,"");

  ame_mass_exp amex;
  o2scl_hdf::ame_load(amex,"");

  mnmsk_mass mth;
  o2scl_hdf::mnmsk_load(mth,"");

  mnmsk_mass_exp mexp;
  o2scl_hdf::mnmsk_load(mexp,"");

  // Test some simple distributions

  simple_dist sd(13,13,15,40,mth);
  for(nuclear_dist::iterator ndi=sd.begin();ndi!=sd.end();ndi++) {
    cout << ndi->Z << " " << ndi->A << " " << ndi->m << endl;
  }
  t.test_gen(sd.size()==20,"sd size");
  
  simple_dist sd2(82,82,208,208,mth);
  nuclear_dist::iterator ndi2=sd2.begin();
  cout << ndi2->mex*hc_mev_fm << " " << ndi2->be/208.0*hc_mev_fm << endl;
  t.test_rel(ndi2->mex*hc_mev_fm,-21.749,5.0e-2,"Pb mex");
  t.test_rel(ndi2->be/208.0*hc_mev_fm,-7.867,5.0e-4,"Pb be");
  
  // Test the size of the distributions

  full_dist fd(mth);
  size_t cnt=0;
  for(nuclear_dist::iterator ndi=fd.begin();ndi!=fd.end();ndi++) {
    cnt++;
  }
  t.test_gen(fd.size()==8979,"full_dist size 1");
  t.test_gen(cnt==8979,"full_dist size 1b");

  full_dist fd2(ame);
  t.test_gen(fd2.size()==3178,"full_dist size 2");
 
  full_dist fd3(amex);
  t.test_gen(fd3.size()==2227,"full_dist size 3");

  full_dist fd4(mexp);
  t.test_gen(fd4.size()==1654,"full_dist size 4");

  t.report();
  return 0;
}
