/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006-2012, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <iostream>
#include <o2scl/test_mgr.h>
#include <o2scl/nuclear_mass.h>
#include <o2scl/hdf_nucmass_io.h>

using namespace std;
using namespace o2scl;
using namespace o2scl_const;
using namespace o2scl_fm;
using namespace o2scl_hdf;

int main(void) {

  test_mgr t;
  t.set_output_level(2);

  cout.setf(ios::scientific);

  // Test parse_elstring()
  {
    nuclear_mass_info nmi;
    int tN, tZ, tA; 
    nmi.parse_elstring("Uuo300",tZ,tN,tA);
    t.test_gen(tZ==118,"parse1");
    t.test_gen(tA==300,"parse2");
    tN=0;
    tZ=0;
    tA=0;
    nmi.parse_elstring("Pb208",tZ,tN,tA);
    t.test_gen(tZ==82,"parse3");
    t.test_gen(tA==208,"parse4");
    tN=0;
    tZ=0;
    tA=0;
    nmi.parse_elstring("Pb-208",tZ,tN,tA);
    t.test_gen(tZ==82,"parse5");
    t.test_gen(tA==208,"parse6");
    tN=0;
    tZ=0;
    tA=0;
    nmi.parse_elstring("Pb 208",tZ,tN,tA);
    t.test_gen(tZ==82,"parse7");
    t.test_gen(tA==208,"parse8");
    tN=0;
    tZ=0;
    tA=0;
    nmi.parse_elstring("pb-208",tZ,tN,tA);
    t.test_gen(tZ==82,"parse9");
    t.test_gen(tA==208,"parse10");
    tN=0;
    tZ=0;
    tA=0;
    nmi.parse_elstring("pb 208",tZ,tN,tA);
    t.test_gen(tZ==82,"parse11");
    t.test_gen(tA==208,"parse12");
    tN=0;
    tZ=0;
    tA=0;
    nmi.parse_elstring("H4",tZ,tN,tA);
    t.test_gen(tZ==1,"parse13");
    t.test_gen(tA==4,"parse14");
    tN=0;
    tZ=0;
    tA=0;
    nmi.parse_elstring("208Pb",tZ,tN,tA);
    t.test_gen(tZ==82,"parse15");
    t.test_gen(tA==208,"parse16");
    tN=0;
    tZ=0;
    tA=0;
    nmi.parse_elstring("208-Pb",tZ,tN,tA);
    t.test_gen(tZ==82,"parse17");
    t.test_gen(tA==208,"parse18");
    tN=0;
    tZ=0;
    tA=0;
    nmi.parse_elstring("208 Pb",tZ,tN,tA);
    t.test_gen(tZ==82,"parse19");
    t.test_gen(tA==208,"parse20");
    tN=0;
    tZ=0;
    tA=0;
    nmi.parse_elstring("208-pb",tZ,tN,tA);
    t.test_gen(tZ==82,"parse21");
    t.test_gen(tA==208,"parse22");
    tN=0;
    tZ=0;
    tA=0;
    nmi.parse_elstring("208 pb",tZ,tN,tA);
    t.test_gen(tZ==82,"parse23");
    t.test_gen(tA==208,"parse24");
    tN=0;
    tZ=0;
    tA=0;
    nmi.parse_elstring("4H",tZ,tN,tA);
    t.test_gen(tZ==1,"parse25");
    t.test_gen(tA==4,"parse26");
  }

  // Create an instance of all the various mass formulae

  ame_mass ame;
  o2scl_hdf::ame_load(ame,"");
  ame_mass ame95rmd;
  o2scl_hdf::ame_load(ame95rmd,"95rmd");
  ame_mass ame03round;
  o2scl_hdf::ame_load(ame03round,"03round");
  ame_mass ame95exp;
  o2scl_hdf::ame_load(ame95exp,"95exp");

  semi_empirical_mass sm;
  
  mnmsk_mass m95;
  o2scl_hdf::mnmsk_load(m95);
  
  dz_mass dz("96");
  dz_mass dz2("95");
  ktuy_mass kt("04");
  ktuy_mass kt2("05");
  
  hfb_mass hfb2;
  o2scl_hdf::hfb_load(hfb2,2);
  hfb_mass hfb8;
  o2scl_hdf::hfb_load(hfb8,8);
  hfb_mass hfb14;
  o2scl_hdf::hfb_load(hfb14,14);

  // Set up generic pointers for testing
  nuclear_mass_disc *nmd[12]={&ame,&ame95rmd,&ame03round,&ame95exp,
			      &m95,&dz,&dz2,&kt,&kt2,&hfb2,&hfb8,
			      &hfb14};

  // Test the spins obtained from mnmsk

  mnmsk_mass_entry mmk;
  mmk=m95.get_ZN(83,125);
  t.test_gen(mmk.spinp==((string)"9/2-"),"spinp");
  t.test_gen(mmk.spinn==((string)"1/2-"),"spinn");

  // Test the various formulae for the binding energy of lead

  t.test_rel(ame.binding_energy(82,126)/208.0,-7.867,1.0e-4,"ame be");
  t.test_rel(sm.binding_energy(82,126)/208.0,-7.867,5.0e-2,"sm be");
  t.test_rel(m95.binding_energy(82,126)/208.0,-7.867,5.0e-4,"m95 be");
  t.test_rel(hfb2.binding_energy(82,126)/208.0,-7.867,2.0e-3,"hfb be");
  t.test_rel(hfb8.binding_energy(82,126)/208.0,-7.867,1.0e-3,"hfb2 be");
  t.test_rel(hfb14.binding_energy(82,126)/208.0,-7.867,1.0e-3,"hfb3 be");
  t.test_rel(dz.binding_energy(82,126)/208.0,-7.867,1.0e-3,"dz be");
  t.test_rel(dz2.binding_energy(82,126)/208.0,-7.867,1.0e-3,"dz2 be");
  t.test_rel(kt.binding_energy(82,126)/208.0,-7.867,1.0e-3,"kt be");
  t.test_rel(kt2.binding_energy(82,126)/208.0,-7.867,1.0e-3,"kt2 be");

  // Test the binding energy and mass excess from get_nucleus()
  for(size_t i=0;i<12;i++) {
    nucleus n;
    nmd[i]->get_nucleus(82,126,n);
    t.test_rel(n.be*o2scl_const::hc_mev_fm/208.0,-7.867,4.0e-3,"ptr be");
    t.test_rel(n.m-126.0*mass_neutron-82.0*mass_proton,n.be,1.0e-12,"ptr be2");
    t.test_rel(n.m+82.0*mass_electron-208.0*mass_amu,n.mex,1.0e-11,
	       "ptr mex");
  }

  // Test size of ame95rmd
  
  t.test_gen(ame95rmd.get_nentries()==2931,"ame.n");

  t.report();
  return 0;
}

