/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006-2012, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifndef O2SCL_REL_BOSON_H
#define O2SCL_REL_BOSON_H

#include <string>
#include <iostream>
#include <fstream>
#include <cmath>
#include <o2scl/constants.h>
#include <o2scl/root.h>
#include <o2scl/mroot.h>
#include <o2scl/inte.h>
#include <o2scl/cern_mroot_root.h>
#include <o2scl/gsl_inte_qag.h>
#include <o2scl/gsl_inte_qagiu.h>

#include <o2scl/boson.h>

#ifndef DOXYGENP
namespace o2scl {
#endif

  /** \brief Equation of state for a relativistic boson

      \todo Testing not completely finished.
  */
  class rel_boson {

  public:

    /// Create a boson with mass \c m and degeneracy \c g
    rel_boson();

    virtual ~rel_boson();

    /// The error value from mroot
    int mroot_err;

    /// The error value from inte
    int inte_err;
    
    /** \brief Calculate properties as function of chemical potential
    */
    virtual void calc_mu(boson &b, double temper);
    
    /** \brief Calculate properties as function of density
    */
    virtual void calc_density(boson &b, double temper);
    
    /** \brief Calculate properties with antiparticles as function of
        chemical potential
    */
    virtual void pair_mu(boson &b, double temper);

    /** \brief Calculate properties with antiparticles as function of
	density
    */
    virtual void pair_density(boson &b, double temper) {
      O2SCL_ERR("Function rel_boson::pair_density() unimplemented.",
		gsl_eunimpl);
    }

    /// Calculate effective chemical potential from density
    virtual void nu_from_n(boson &b, double temper);

    /// Set inte object
    void set_inte(inte<funct > &l_nit, inte<funct > &l_dit);

    /** \brief Set the solver for use in calculating the chemical
	potential from the density */
    void set_density_root(root<funct > &rp) {
      density_root=&rp;
      return;
    }

    /// The default solver for calc_density().
    cern_mroot_root<funct > def_density_root;

    /// Default nondegenerate integrator
    gsl_inte_qagiu<funct > def_nit;

    /// Default degenerate integrator
    gsl_inte_qag<funct > def_dit;

    /// Return string denoting type ("rel_boson")
    virtual const char *type() { return "rel_boson"; }

  protected:

#ifndef DOXYGENP

    /// The non-degenerate integrator
    inte<funct> *nit;
    /// The degenerate integrator
    inte<funct> *dit;
    /// The solver for calc_density()
    root<funct> *density_root;

    /// Desc
    boson *bp;

    /// Desc
    double T;
    
    /// Non-degenerate density integral
    double density_fun(double u);
    /// Non-degenerate energy density integral
    double energy_fun(double u);
    /// Non-degenerate entropy integral
    double entropy_fun(double u);
    /// Degenerate density integral
    double deg_density_fun(double u);
    /// Degenerate energy density integral
    double deg_energy_fun(double u);
    /// Degenerate entropy integral
    double deg_entropy_fun(double u);
    /// Solve for the density in calc_density()
    double solve_fun(double x);

#endif

  };

#ifndef DOXYGENP
}
#endif

#endif
